using System;
using System.Collections;
using System.IO;
using Server;
using Server.Items;

namespace Server.Scripts.Commands
{
	public class SignParser
	{
		private class SignEntry
		{
			public string m_Text;
			public Point3D m_Location;
			public int m_ItemID;

			public SignEntry( string text, Point3D pt, int itemID )
			{
				m_Text = text;
				m_Location = pt;
				m_ItemID = itemID;
			}

			public override bool Equals( object obj )
			{
				if ( !(obj is SignEntry) ) return false;

				SignEntry e = (SignEntry)obj;

				return m_Text == e.m_Text && m_Location == e.m_Location && m_ItemID == e.m_ItemID;
			}

			public override int GetHashCode()
			{
				return m_Location.GetHashCode();
			}

			public override string ToString()
			{
				return String.Format( "\"{0}\" {1}", m_Text, m_Location );
			}
		}

		public static void Initialize()
		{
			Server.Commands.Register( "SignGen", AccessLevel.Administrator, new CommandEventHandler( SignGen_OnCommand ) );
		}

		public static void SignGen_OnCommand( CommandEventArgs c )
		{
			Parse( c.Mobile );
		}

		public static void Parse( Mobile from )
		{
			string cfg = Path.Combine( Core.BaseDirectory, "signs.cfg" );
			if ( File.Exists( cfg ) )
			{
				ArrayList list = new ArrayList();
				from.SendMessage( "Generating signs..." );

				using ( StreamReader ip = new StreamReader( cfg ) )
				{
					string line;

					while ( (line = ip.ReadLine()) != null )
					{
						string[] split = line.Split( ' ' );

						SignEntry e = new SignEntry( line.Substring( split[0].Length + 1 + split[1].Length + 1 + split[2].Length + 1 + split[3].Length + 1 ), new Point3D( Utility.ToInt32( split[1] ), Utility.ToInt32( split[2] ), Utility.ToInt32( split[3] ) ), Utility.ToInt32( split[0] ) );

						if ( !list.Contains( e ) )
						{
							list.Add( e );
						}
					}
				}

				int i = 0;

				while ( i < list.Count )
				{
					SignEntry e = (SignEntry)list[i];

					int j = i + 1;

					int dupes = 0;

					while ( j < list.Count )
					{
						SignEntry test = (SignEntry)list[j];

						if ( e.m_Location == test.m_Location )
						{
							if ( dupes++ == 0 )
							{
								Console.WriteLine( "Dupes:" );
								Console.WriteLine( " - #1 : {0}", e );
							}

							Console.WriteLine( " - #{0} : {1}", dupes+1, test );
						}

						++j;
					}

					Item sign = null;

					if ( dupes > 0 )
					{
						Console.Write( "Use which one? #" );
						int index = Utility.ToInt32( Console.ReadLine() );

						index -= 1;
						if ( index < 0 )
							index = 0;

						j = i;
						dupes = 0;

						while ( j < list.Count )
						{
							SignEntry test = (SignEntry)list[j];

							if ( e.m_Location == test.m_Location )
							{
								if ( dupes++ == index )
								{
									sign = new Sign( test.m_ItemID );
									sign.MoveToWorld( test.m_Location, from.Map );
									sign.Name = test.m_Text;
									sign.Movable = false;
								}

								i = j + 1;
							}

							++j;
						}
					}
					else
					{
						sign = new Sign( e.m_ItemID );
						sign.MoveToWorld( e.m_Location, from.Map );
						sign.Name = e.m_Text;

						++i;
					}

					/*if ( sign != null )
					{
						bool hasSignpost = false;

						foreach ( Tile t in sign.Map.GetTilesAt( new Point2D( sign.X, sign.Y ), false, false, true ) )
						{
							if ( t.ID >= 0x4B97 && t.ID <= 0x4BA2 )
							{
								hasSignpost = true;
								break;
							}
						}

						if ( !hasSignpost )
						{
							from.Location = sign.Location;

							Console.WriteLine( "Sign named {0} found without signpost : accept? (y/n)", sign.Name );

							if ( Console.ReadLine() == "n" )
							{
								newList.RemoveAt( newList.Count - 1 );
								sign.Delete();
							}
						}
					}*/
				}

				from.SendMessage( "Done!" );
			}
			else
			{
				from.SendMessage( "{0} not found!", cfg );
			}
		}
	}
}