// Carpentry Definition / Craft System Script v.0.2 by SickLab

using System;
using Server.Items;

namespace Server.Engines.Craft
{
	public class DefCarpentry : CraftSystem
	{
		public override SkillName MainSkill
		{
			get	{ return SkillName.Carpentry;	}
		}

		public override string GumpTitle
		{
			get { return "CARPENTRY MENU"; }
		}

		public override int GumpTitlePos
		{
			get { return 175; }
		}

		private static CraftSystem m_CraftSystem;

		public static CraftSystem CraftSystem
		{
			get
			{
				if ( m_CraftSystem == null )
					m_CraftSystem = new DefCarpentry();

				return m_CraftSystem;
			}
		}

		private DefCarpentry() : base(1, 2, 3.0)
		{
		}

		public override bool CanCraft( Mobile from, BaseTool tool, Type itemType )
		{
			if ( tool.Deleted )
			{
				return false;
			}
			else
			{
				if ( tool.Durability < 1 )
				{
					from.SendMessage( "This tool is broken" );
					return false;
				}
			}

			return true;
		}

		public override void PlayCraftEffect( Mobile from )
		{
			if ( from.Body.Type == BodyType.Human && !from.Mounted )
				from.Animate( 9, 5, 1, true, false, 0 );

			from.PlaySound( 0x23D );
		}

		public override void PlayEndingEffect( Mobile from, bool failed, bool lostMaterial, bool toolBroken, int quality )
		{
			if ( toolBroken )
			{
				from.SendLocalizedMessage( 1044038 ); // You have worn out your tool
			}

			if ( failed )
			{
				if ( lostMaterial) 
				{
					from.SendLocalizedMessage( 1044043 ); // You failed to create the item, and some of your materials are lost.
				}
				else
				{
					from.SendLocalizedMessage( 1044157 ); // You failed to create the item, but no materials were lost.
				}
			}
			else
			{
				if ( quality == 0 )
					from.SendLocalizedMessage( 502785 ); // You were barely able to make the item.  It's quality is below average
				else if (quality == 2 )
					from.SendLocalizedMessage( 502784 ); // Due to your exceptional skill, its quality is higher than average
				else				
					from.SendLocalizedMessage( 1044154 ); // You create the item
			}
		}

		public override void InitCraftList()
		{
			int index = -1;
			
			// Other Items
			AddCraft( typeof( Board ), "Other Items", "Board", 0.0, 20.0, typeof( Log ), "Log", 1 );
			AddCraft( typeof( BarrelStaves ), "Other Items", "Barrel Staves", 0.0, 20.0, typeof( Log ), "Log", 5 );
			AddCraft( typeof( BarrelLid ), "Other Items", "Barrel Lid", 11.0, 35.0, typeof( Log ), "Log", 4 );
			AddCraft( typeof( ShortMusicStand ), "Other Items", "Short Music Stand", 78.9, 100.0, typeof( Log ), "Log", 15 );
			AddCraft( typeof( TallMusicStand ), "Other Items", "Tall Music Stand", 81.5, 100.0, typeof( Log ), "Log", 20 );
			AddCraft( typeof( Easle ), "Other Items", "Easle", 86.8, 100.0, typeof( Log ), "Log", 20 );

			// Furniture
			AddCraft( typeof( FootStool ), "Furniture", "Foot Stool", 11.0, 30.0, typeof( Log ), "Log", 9 );
			AddCraft( typeof( Stool ), "Furniture", "Stool", 11.0, 30.0, typeof( Log ), "Log", 9 );
			
			index = AddCraft( typeof( FancyWoodenChairCushion ), "Furniture", "Fancy Wooden Chair Cushion", 55.0, 70.0, typeof( Log ), "Log", 15 );
			AddSkill( index, SkillName.Tailoring, 20.0, 40.0 );
			AddRes( index, typeof( Cloth ), "Cloth", 5 );
			
			index = AddCraft( typeof( WoodenChairCushion ), "Furniture", "Wooden Chair Cushion", 21.0, 40.0, typeof( Log ), "Log", 13 );
			AddSkill( index, SkillName.Tailoring, 15.0, 30.0 );
			AddRes( index, typeof( Cloth ), "Cloth", 5 );
			
			AddCraft( typeof( WoodenChair ), "Furniture", "Wooden Chair", 21.0, 40.0, typeof( Log ), "Log", 13 );
			AddCraft( typeof( BambooChair ), "Furniture", "Bamboo Chair", 21.0, 40.0, typeof( Log ), "Log", 13 );
			AddCraft( typeof( WoodenBench ), "Furniture", "Wooden Bench", 52.6, 75.0, typeof( Log ), "Log", 17 );
			AddCraft( typeof( Throne ), "Furniture", "Throne", 73.6, 100.0, typeof( Log ), "Log", 19 );
			AddCraft( typeof( WoodenThrone ), "Furniture", "Wooden Throne", 52.6, 75.0, typeof( Log ), "Log", 17 );
			AddCraft( typeof( Nightstand ), "Furniture", "Nightstand", 42.1, 65.0, typeof( Log ), "Log", 17 );
			AddCraft( typeof( LargeTable ), "Furniture", "Large Table", 63.1, 85.0, typeof( Log ), "Log", 23 );
			AddCraft( typeof( WritingTable ), "Furniture", "Writing Table", 63.1, 85.0, typeof( Log ), "Log", 17 );
			AddCraft( typeof( YewWoodTable ), "Furniture", "Yew-Wood Table", 84.2, 100.0, typeof( Log ), "Log", 27 );

			//Staves
			AddCraft( typeof( ShepherdsCrook ), "Staves", "Shepherd's Crook", 78.9, 100.0, typeof( Log ), "Log", 7 );
			AddCraft( typeof( QuarterStaff ), "Staves", "Quarter Staff", 73.6, 100.0, typeof( Log ), "Log", 6 );
			AddCraft( typeof( GnarledStaff ), "Staves", "Gnarled Staff", 78.9, 100.0, typeof( Log ), "Log", 7 );
			AddCraft( typeof( BlackStaff ), "Staves", "Black Staff", 85.9, 100.0, typeof( Log ), "Log", 7 );

			// Instruments
			index = AddCraft( typeof( LapHarp ), "Instruments", "Lap Harp", 63.1, 100.0, typeof( Log ), "Log", 20 );
			AddSkill( index, SkillName.Musicianship, 45.0, 50.0 );
			AddRes( index, typeof( Cloth ), "Cloth", 10 );

			index = AddCraft( typeof( Harp ), "Instruments", "Standing Harp", 78.9, 100.0, typeof( Log ), "Log", 35 );
			AddSkill( index, SkillName.Musicianship, 45.0, 50.0 );
			AddRes( index, typeof( Cloth ), "Cloth", 15 );
			
			index = AddCraft( typeof( Drums ), "Instruments", "Drum", 57.8, 100.0, typeof( Log ), "Log", 20 );
			AddSkill( index, SkillName.Musicianship, 45.0, 50.0 );
			AddRes( index, typeof( Cloth ), "Cloth", 10 );
			
			index = AddCraft( typeof( Lute ), "Instruments", "Lute", 68.4, 100.0, typeof( Log ), "Log", 25 );
			AddSkill( index, SkillName.Musicianship, 45.0, 50.0 );
			AddRes( index, typeof( Cloth ), "Cloth", 10 );
			
			index = AddCraft( typeof( Tambourine ), "Instruments", "Tambourine", 57.8, 100.0, typeof( Log ), "Log", 15 );
			AddSkill( index, SkillName.Musicianship, 45.0, 50.0 );
			AddRes( index, typeof( Cloth ), "Cloth", 10 );

			index = AddCraft( typeof( TambourineTassel ), "Instruments", "Tambourine (tassel)", 57.8, 100.0, typeof( Log ), "Log", 15 );
			AddSkill( index, SkillName.Musicianship, 45.0, 50.0 );
			AddRes( index, typeof( Cloth ), "Cloth", 15 );			
		}
	}
}