using System;
using Server;
using Server.Gumps;
using Server.Network;
using Server.Mobiles;

namespace Server.Gumps
{
	public enum ResurrectMessage
	{
		ChaosShrine = 0,
		VirtueShrine = 1,
		Healer = 2,
		Generic = 3,
	}

	public class ResurrectGump : Gump
	{
		public ResurrectGump( Mobile owner ) : this( owner, ResurrectMessage.Generic )
		{
		}

		public ResurrectGump( Mobile owner, ResurrectMessage msg ) : base( 0, 0 )
		{
			Closable = false;
			Resizable = false;

			AddBackground( 265, 205, 320, 290, 5054 );        
			
			AddPage( 0 );      

			AddHtmlLocalized( 267, 195, 310, 140, 1011022, false, false );  // Ressurection
			
			AddImageTiled( 225, 175, 50, 45, 0xCE );   //Top left corner
			AddImageTiled( 267, 175, 315, 44, 0xC9 );  //Top bar
			AddImageTiled( 582, 175, 43, 45, 0xCF );   //Top right corner
			AddImageTiled( 225, 219, 44, 270, 0xCA );  //Left side
			AddImageTiled( 582, 219, 44, 270, 0xCB );  //Right side
			AddImageTiled( 225, 489, 44, 43, 0xCC );   //Lower left corner
			AddImageTiled( 267, 489, 315, 43, 0xE9 );  //Lower Bar
			AddImageTiled( 582, 489, 43, 43, 0xCD );   //Lower right corner

			//1011023 - It is possible for you to be resurrected here at the chaos shrine. Do you wish to try? (Continue to try, Cancel to not, blah blah)
			//1011024 - It is possible for you to be resurrected here at this shrine of the virtues. Do you wish to try? (Continue to try, Cancel to not, blah blah)
			//1011025 - It is possible for you to be resurrected by this healer. Do you wish to try? (Continue to try, Cancel to not, blah blah)
			//1011026 - It is possible for you to be ressurected now. Do you wish to try? (Continue to try, Cancel to not, blah blah)
			AddHtmlLocalized( 267, 234, 300, 140, 1011023 + ((int)msg), true, true );

			AddPage( 1 );

			AddButton( 290, 405, 0xFA5, 0xFA7, 1, GumpButtonType.Reply, 0 );
			AddHtmlLocalized( 325, 408, 300, 50, 1011011, false, false ); // Continue
			      	
			AddButton( 427, 405, 0xFA5, 0xFA7, 2, GumpButtonType.Reply, 0 );
			AddHtmlLocalized( 462, 408, 300, 50, 1011012, false, false ); // Cancel		

			owner.Frozen = true;//freeze until they choose
		}

		public override void OnResponse( NetState state, RelayInfo info )
		{
			Mobile from = state.Mobile;

			from.Frozen = false;
			from.CloseGump( typeof( ResurrectGump ) );

			switch ( info.ButtonID )
			{
				case 1: //continue
				{
					from.Resurrect();
					if ( from.ShortTermMurders >= 5 )
					{
						double loss = (100.0 - ( 4.0 + (from.ShortTermMurders/5.0))) / 100.0;//5 to 15% loss
						if ( loss < 0.85 )
							loss = 0.85;
						else if ( loss > 0.95 )
							loss = 0.95;

						if ( from.Str * loss > 10 )
							from.Str = (int)(from.Str * loss);
						if ( from.Int * loss > 10 )
							from.Int = (int)(from.Int * loss);
						if ( from.Dex * loss > 10 )
							from.Dex = (int)(from.Dex * loss);

						for (int s=0;s<from.Skills.Length;s++)
						{
							if ( from.Skills[s].Base * loss > 35 )
								from.Skills[s].Base *= loss;
						}
					}
					break; 
				}
				case 2: //cancel
				{
					break; 
				}
			}
		}
	}
}
