using System;
using Server;

namespace Server.Items
{
	public class Teleporter : Item
	{
		private bool m_Active;
		private Point3D m_PointDest;
		private Map m_MapDest;

		[CommandProperty( AccessLevel.GameMaster )]
		public bool Active
		{
			get { return m_Active; }
			set { m_Active = value; }
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public Point3D PointDest
		{
			get { return m_PointDest; }
			set { m_PointDest = value; }
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public Map MapDest
		{
			get { return m_MapDest; }
			set { m_MapDest = value; }
		}

		[Constructable]
		public Teleporter() : this( new Point3D(0, 0, 0), null )
		{
		}

		[Constructable]
		public Teleporter( Point3D pointDest, Map mapDest ) : base( 0x1BC3 )
		{
			Movable = false;
			Visible = false;

			Name = "teleporter";

			m_Active = true;
			m_PointDest = pointDest;
			m_MapDest = mapDest;
		}

		public override void OnSingleClick( Mobile from )
		{
			LabelTo( from, 1026095 ); // teleporter

			if ( m_Active )
			{
				if ( m_MapDest != null && m_PointDest != Point3D.Zero )
					LabelTo( from, "{0} [{1}]", m_PointDest, m_MapDest );
				else if ( m_MapDest != null )
					LabelTo( from, "[{0}]", m_MapDest );
				else if ( m_PointDest != Point3D.Zero )
					LabelTo( from, m_PointDest.ToString() );
			}
			else
			{
				LabelTo( from, "(inactive)" );
			}
		}

		public override bool OnMoveOver( Mobile m )
		{
			if ( m_Active )
			{
				if ( m_MapDest != null )
					m.Map = m_MapDest;

				if ( m_PointDest != Point3D.Zero )
					m.Location = m_PointDest;

				return false;
			}

			return true;
		}

		public Teleporter( Serial serial ) : base( serial )
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version

			writer.Write( m_Active );
			writer.Write( m_PointDest );
			writer.Write( m_MapDest );
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();

			switch ( version )
			{
				case 0:
				{
					m_Active = reader.ReadBool();
					m_PointDest = reader.ReadPoint3D();
					m_MapDest = reader.ReadMap();

					break;
				}
			}
		}
	}
}
