using System;
using System.Collections;
using Server.Misc;
using Server.Items;

namespace Server.Mobiles
{
	[Dispellable]
	[CorpseName( "a blade spirit corpse" )]
	public class BladeSpirits : BaseCreature
	{
		private Timer m_Timer;

		[Constructable]
		public BladeSpirits() : base( AIType.AI_Melee, FightMode.Closest, 10, 1, 0.3, 0.6 )
		{
			Body = 574;
			this.Kills = 500; // Murderer
			Name = "a blade spirit";

			// TODO: 1200 hits max, 250 stamina

			Team = Utility.Random( Int32.MaxValue - 2 );

			SetStr( 125, 175 );
			SetDex( 125, 175 );
			SetInt( 125, 175 );

			SetSkill( SkillName.Swords, 91 );
			SetSkill( SkillName.Tactics, 90 );
			SetSkill( SkillName.MagicResist, 70 );

			m_Timer = new ExpireTimer( this );
			m_Timer.Start();

			Katana kat = new Katana();

			kat.Newbied = true;
			kat.Movable = false;
			kat.Quality = WeaponQuality.Exceptional;

			AddItem( kat );
		}

		public BladeSpirits( Serial serial ) : base( serial )
		{
			m_Timer = new ExpireTimer( this );
			m_Timer.Start();
		}

		public override void OnAfterDelete()
		{
			m_Timer.Stop();

			base.OnAfterDelete();
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}

		private class ExpireTimer : Timer
		{
			private BladeSpirits m_Owner;

			public ExpireTimer( BladeSpirits owner ) : base( TimeSpan.FromMinutes( 1.5 ) )
			{
				this.Priority = TimerPriority.FiveSeconds;

				m_Owner = owner;
			}

			protected override void OnTick()
			{
				m_Owner.Delete();
			}
		}
	}
}
