using System;
using System.Collections;
using Server.Misc;
using Server.Spells;
using Server.Spells.First;
using Server.Spells.Second;
using Server.Spells.Third;
using Server.Spells.Fourth;
using Server.Spells.Fifth;
using Server.Spells.Sixth;
using Server.Spells.Seventh;
using Server.Targeting;

namespace Server.Mobiles
{
	[Dispellable]
	[CorpseName( "an energy vortex corpse" )]
	public class EnergyVortex : BaseCreature
	{
		private Timer m_Timer;

		[Constructable]
		public EnergyVortex() : base( AIType.AI_Melee, FightMode.Closest, 10, 1, 0.2, 0.4 )
		{
			Body = 164;
			this.Kills = 500; // Murderer
			Name = "an energy vortex";
			BaseSoundID = 263;

			SetStr( 450, 550 );
			SetDex( 400, 500 );
			SetInt( 450, 550 );

			Team = Utility.Random( Int32.MaxValue - 2 );

			SetSkill( SkillName.Wrestling, 120 );
			SetSkill( SkillName.Tactics, 100 );
			SetSkill( SkillName.Anatomy, 100 );
			SetSkill( SkillName.MagicResist, 100 );

			m_Timer = new ExpireTimer( this );
			m_Timer.Start();
		}

		public EnergyVortex( Serial serial ) : base( serial )
		{
			m_Timer = new ExpireTimer( this );
			m_Timer.Start();
		}

		public override void OnAfterDelete()
		{
			m_Timer.Stop();

			base.OnAfterDelete();
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}

		private class ExpireTimer : Timer
		{
			private EnergyVortex m_Owner;

			public ExpireTimer( EnergyVortex owner ) : base( TimeSpan.FromMinutes( 1.5 ) )
			{
				this.Priority = TimerPriority.FiveSeconds;

				m_Owner = owner;
			}

			protected override void OnTick()
			{
				m_Owner.Delete();
			}
		}
	}
}
