using System;
using Server;
using Server.Multis;

namespace Server.Items
{
	public class TillerMan : Item
	{
		private Boat m_Boat;

		public TillerMan( Boat boat ) : base( 0x3E4E )
		{
			m_Boat = boat;
			Movable = false;
		}

		public TillerMan( Serial serial ) : base(serial)
		{
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 );//version

			writer.Write( m_Boat );
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();

			switch ( version )
			{
				case 0:
				{
					m_Boat = reader.ReadItem() as Boat;
					break;
				}
			}
		}

		public void Turn( Direction dir )
		{
			switch ( dir )
			{
				case Direction.North:
					ItemID = 0x3E4E;
					break;
				case Direction.East:
					ItemID = 0x3E53;
					break;
				case Direction.South:
					ItemID = 0x3E4B;
					break;
				case Direction.West:
					ItemID = 0x3E50;
					break;
			}
		}

		public override void OnAfterDelete()
		{
			if ( m_Boat != null && !m_Boat.Deleted )
				m_Boat.Delete();
		}

		public override void OnSpeech( SpeechEventArgs e )
		{
			if ( e.Mobile.InRange( this, 8 ) && m_Boat.Contains( e.Mobile ) )
			{
				if ( e.HasKeyword( 0x45 ) )//forward
				{
					m_Boat.StartMove( 2 );
				}
				else if ( e.HasKeyword( 0x46 ) )//backwards
				{
					int dir = (int)m_Boat.Direction + 4;
					if ( dir > 7 )
						dir %= 8;

					m_Boat.StartMove( (Direction)dir, 2 );
				}
				else if ( e.HasKeyword( 0x47 ) )//left / drift left
				{
					int dir = (int)m_Boat.Direction - 2;
					if ( dir < 0 )
						dir += 8;
					m_Boat.StartMove( (Direction)dir, 2 );
				}
				else if ( e.HasKeyword( 0x48 ) )//right / drift right
				{
					int dir = (int)m_Boat.Direction + 2;
					if ( dir > 7 )
						dir %= 8;
					m_Boat.StartMove( (Direction)dir, 2 );
				}
					/*else if ( e.HasKeyword( 0x49 ) )//starboard
					{
					}
					else if ( e.HasKeyword( 0x4A ) )//port
					{
					}*/
				else if ( e.HasKeyword( 0x4B ) )//foreward left
				{
					int dir = (int)m_Boat.Direction - 1;
					if ( dir < 0 )
						dir += 8;

					m_Boat.StartMove( (Direction)dir, 2 );
				}
				else if ( e.HasKeyword( 0x4C ) )//foreward right
				{
					int dir = (int)m_Boat.Direction + 1;
					if ( dir > 7 )
						dir %= 8;

					m_Boat.StartMove( (Direction)dir, 2 );
				}
				else if ( e.HasKeyword( 0x4D ) )//back left
				{
					int dir = (int)m_Boat.Direction - 1 + 4;
					if ( dir > 7 )
						dir %= 8;

					m_Boat.StartMove( (Direction)dir, 2 );
				}
				else if ( e.HasKeyword( 0x4E ) )//back right
				{
					int dir = (int)m_Boat.Direction + 1 + 4;
					if ( dir > 7 )
						dir %= 8;

					m_Boat.StartMove( (Direction)dir, 2 );
				}
				else if ( e.HasKeyword( 0x4F ) )//stop
				{
					m_Boat.StopMove();
				}
				else if ( e.HasKeyword( 0x50 ) )//slow left
				{
					int dir = (int)m_Boat.Direction - 2;
					if ( dir < 0 )
						dir += 8;

					m_Boat.StartMove( (Direction)dir, 1 );
				}
				else if ( e.HasKeyword( 0x51 ) )//slow right
				{
					int dir = (int)m_Boat.Direction + 2;
					if ( dir > 7 )
						dir %= 8;

					m_Boat.StartMove( (Direction)dir, 1 );
				}
				else if ( e.HasKeyword( 0x52 ) )//slow foreward
				{
					m_Boat.StartMove( 1 );
				}
				else if ( e.HasKeyword( 0x53 ) )//slow back
				{
					int dir = (int)m_Boat.Direction + 4;
					if ( dir > 7 )
						dir %= 8;

					m_Boat.StartMove( (Direction)dir, 1 );
				}
				else if ( e.HasKeyword( 0x54 ) )//slow foreward left
				{
					int dir = (int)m_Boat.Direction - 1;
					if ( dir < 0 )
						dir += 8;

					m_Boat.StartMove( (Direction)dir, 1 );
				}
				else if ( e.HasKeyword( 0x55 ) )//slow foreward right
				{
					int dir = (int)m_Boat.Direction + 1;
					if ( dir > 7 )
						dir %= 8;

					m_Boat.StartMove( (Direction)dir, 1 );
				}
				else if ( e.HasKeyword( 0x56 ) )//slow back left
				{
					int dir = (int)m_Boat.Direction - 1 + 4;
					if ( dir > 7 )
						dir %= 8;

					m_Boat.StartMove( (Direction)dir, 1 );
				}
				else if ( e.HasKeyword( 0x57 ) )//slow back right
				{
					int dir = (int)m_Boat.Direction + 1 + 4;
					if ( dir > 7 )
						dir %= 8;

					m_Boat.StartMove( (Direction)dir, 1 );
				}
				else if ( e.HasKeyword( 0x58 ) )//left one
				{
					int dir = (int)m_Boat.Direction - 2;
					if ( dir < 0 )
						dir += 8;

					m_Boat.Move( (Direction)dir, 1 );
				}
				else if ( e.HasKeyword( 0x59 ) )//right one
				{
					int dir = (int)m_Boat.Direction + 2;
					if ( dir > 7 )
						dir %= 8;

					m_Boat.Move( (Direction)dir, 1 );
				}
				else if ( e.HasKeyword( 0x5A ) )//foreward one
				{
					m_Boat.Move( 1 );
				}
				else if ( e.HasKeyword( 0x5B ) )//back one
				{
					int dir = (int)m_Boat.Direction + 4;
					if ( dir > 7 )
						dir %= 8;

					m_Boat.Move( (Direction)dir, 1 );
				}
				else if ( e.HasKeyword( 0x65 ) )//turn right
				{
					int dir = (int)m_Boat.Direction + 2;
					if ( dir > 7 )
						dir %= 8;
					m_Boat.SetDirection( (Direction)dir );
				}
				else if ( e.HasKeyword( 0x66 ) )//turn left
				{
					int dir = (int)m_Boat.Direction - 2;
					if ( dir < 0 )
						dir += 8;
					m_Boat.SetDirection( (Direction)dir );
				}
				else if ( e.HasKeyword( 0x67 ) )//turn around
				{
					int dir = (int)m_Boat.Direction + 4;
					if ( dir > 7 )
						dir %= 8;
					m_Boat.SetDirection( (Direction)dir );
				}
			}
		}
	}
}
