using System;
using Server.Targeting;
using Server.Network;

namespace Server.Spells.Fifth
{
	public class MindBlastSpell : Spell
	{
		private static SpellInfo m_Info = new SpellInfo(
				"Mind Blast", "Por Corp Wis",
				SpellCircle.Fifth,
				218,
				Reagent.BlackPearl,
				Reagent.MandrakeRoot,
				Reagent.Nightshade,
				Reagent.SulfurousAsh
			);

		public MindBlastSpell( Mobile caster, Item scroll ) : base( caster, scroll, m_Info )
		{
		}

		public override void OnCast()
		{
			Caster.Target = new InternalTarget( this );
		}

		public void Target( Mobile m )
		{
			if ( !Caster.CanSee( m ) )
			{
				Caster.SendLocalizedMessage( 500237 ); // Target can not be seen.
			}
			else if ( CheckHSequence( m ) )
			{
				// Algorithm: (highestStat - lowestStat) / 2 [- 50% if resisted]

				SpellHelper.CheckReflect( (int)this.Circle, Caster, ref m );

				int highestStat = m.Str, lowestStat = m.Str;

				if ( m.Dex > highestStat )
					highestStat = m.Dex;

				if ( m.Dex < lowestStat )
					lowestStat = m.Dex;

				if ( m.Int > highestStat )
					highestStat = m.Int;

				if ( m.Int < lowestStat )
					lowestStat = m.Int;

				if ( highestStat > 150 )
					highestStat = 150;

				if ( lowestStat > 150 ) 
					lowestStat = 150;

				int damage = (highestStat - lowestStat) / 4;//less damage

				if ( damage > 45 )
					damage = 45;

				if ( CheckResisted( m ) )
				{
					damage /= 2;
					m.SendLocalizedMessage( 501783 ); // You feel yourself resisting magical energy.
				}

				if ( Caster != m )
					Caster.Direction = Caster.GetDirectionTo( m );

				m.PlaySound( 0x213 );
				Effects.SendTargetEffect( m, 0x374A, 16 );

				new SpellHelper.SpellDamageTimer( this, m, damage ).Start();
			}

			FinishSequence();
		}

		private class InternalTarget : Target
		{
			private MindBlastSpell m_Owner;

			public InternalTarget( MindBlastSpell owner ) : base( 12, true, TargetFlags.Harmful )
			{
				m_Owner = owner;
			}

			protected override void OnTarget( Mobile from, object o )
			{
				if ( o is Mobile )
				{
					m_Owner.Target( (Mobile)o );
				}
			}

			protected override void OnTargetFinish( Mobile from )
			{
				m_Owner.FinishSequence();
			}
		}
	}
}